@@ -835,7 +835,19 @@ copyFromRobjectViaActiveBindings <- function(Robj, cppNames, cppCopyTypes, .self
     }
 }
 
-copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll) {
+copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll,
+                            useCompiledCopyMethod = FALSE) {
+    if(useCompiledCopyMethod) {
+        ## Copy some elements from C++ copyFromRobject method
+        ## Currently this includes various numeric types as well as and nodeFxnVector
+        ## There is a problem creating the C++ method for predefined nimbleLists,
+        ##   so currently useCompiledCopyMethod will be FALSE for (all) nimbleLists
+
+        ## use eval() to avoid R CMD check issue with registration 
+        eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$copyFromRobject, basePtr, Robj))
+        ## .Call(nimbleUserNamespace$sessionSpecificDll$copyFromRobject, basePtr, Robj)
+    }
+    
     for(v in cppNames) {
         if(is.null(cppCopyTypes[[v]])) next
         if(is.null(Robj[[v]])) {
@@ -847,10 +859,12 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
             modelVar <- Robj[[v]] ## this is a singleVarAccessClass created by replaceModelSingles
             Cmodel <- modelVar$model$CobjectInterface
             varName <- modelVar$var
-            ##message('copying modelVar (copyFromRobject)')
             getSetModelVarPtr(v, eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getModelObjectPtr, Cmodel$.basePtr, varName)), basePtr, dll = dll)
         },
         'nimbleFunction' = {
+            if(useCompiledCopyMethod) {
+                NULL
+            } else {
             modelVar <- Robj[[v]]
             Cnf <- nf_getRefClassObject(modelVar)$.CobjectInterface ##environment(modelVar)$.CobjectInterface
             if(is.list(Cnf)) {
@@ -859,6 +873,7 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
                 valueBasePtr <- Cnf$.basePtr
             }
             getSetNimbleFunction(v, valueBasePtr, basePtr, dll = dll)
+            }
         },
         'nimbleList' = {
             modelVar <- Robj[[v]]
@@ -895,7 +910,11 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
             getSetModelValues(v, Cmv, basePtr, dll = dll)
         },
         'nodeFxnVec' = {
-            populateNodeFxnVecNew(fxnPtr = basePtr, Robject = Robj, fxnVecName = v, dll = dll)
+            if(useCompiledCopyMethod) {
+                NULL
+            } else {
+                populateNodeFxnVecNew(fxnPtr = basePtr, Robject = Robj, fxnVecName = v, dll = dll)
+            }
         },
         'modelVarAccess' = {
             populateManyModelVarMapAccess(fxnPtr = basePtr, Robject = Robj, manyAccessName = v, dll = dll)
@@ -921,16 +940,32 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
             getSetCharacterScalar(v, Robj[[v]], basePtr, dll = dll)
         },
         'numericVector' = {
-            getSetNumericVector(v, Robj[[v]], basePtr, dll = dll)
+        if(useCompiledCopyMethod) {
+                NULL
+            } else {
+                getSetNumericVector(v, Robj[[v]], basePtr, dll = dll)
+            }
         },
         'doubleScalar' = {
-            getSetDoubleScalar(v, Robj[[v]], basePtr, dll = dll)
+         if(useCompiledCopyMethod) {
+                NULL
+            } else {
+                getSetDoubleScalar(v, Robj[[v]], basePtr, dll = dll)
+            }
         },
         'integerScalar' = {
-            getSetIntegerScalar(v, Robj[[v]], basePtr, dll = dll)
+        if(useCompiledCopyMethod) {
+                NULL
+            } else {
+                getSetIntegerScalar(v, Robj[[v]], basePtr, dll = dll)
+            }
         },
         'logicalScalar' = {
-            getSetLogicalScalar(v, Robj[[v]], basePtr, dll = dll)
+            if(useCompiledCopyMethod) {
+                NULL
+            } else {
+                getSetLogicalScalar(v, Robj[[v]], basePtr, dll = dll)
+            }
         },
         { ## default:
             if(!(cppCopyTypes[[v]] %in% c('copierVector'))) {
@@ -1178,13 +1213,16 @@ CmultiNimbleObjClass <- setRefClass('CmultiNimbleObjClass',
                                                   finalizationPtrList = 'ANY',
                                                   cppNames = 'ANY',
                                                   cppCopyTypes = 'ANY',
+                                                  cppNamesOneByOne = 'ANY',
+                                                  cppCopyTypesOneByOne = 'ANY',
                                                   basePtrCall = 'ANY',
                                                   dll = 'ANY',
                                                   RobjectList = 'ANY',
                                                   compiledNodeFun = 'ANY'    ## a cppNimbleFunctionClass or cppNimbleListClass
                                                   ),
                                     methods = list(
-                                        initialize = function(compiledNodeFun, basePtrCall, project, ...) { ## need to set dll, nimbleProject
+                                        initialize = function(compiledNodeFun, basePtrCall, ##copyFromRobjectCall,
+                                                              project, ...) { ## need to set dll, nimbleProject
                                             nimbleProject <<- project
                                             finalizationPtrList <<- list()
                                             RobjectList <<- list()
@@ -1198,7 +1236,7 @@ CmultiNimbleObjClass <- setRefClass('CmultiNimbleObjClass',
                                             cppCopyTypes <<- makeNimbleFxnCppCopyTypes(symTab, cppNames)
                                         },
                                         finalize = function() {
-                                            for(i in seq_along(finalizationPtrList)) { ## previously basePtrList
+                                            for(i in seq_along(finalizationPtrList)) { 
                                                 if(!is.null(finalizationPtrList[[i]])) {
                                                     nimbleInternalFunctions$nimbleFinalize(finalizationPtrList[[i]])
                                                 }
@@ -1239,7 +1277,6 @@ CmultiNimbleObjClass <- setRefClass('CmultiNimbleObjClass',
 CmultiNimbleFunctionClass <- setRefClass('CmultiNimbleFunctionClass',
                                          contains = 'CmultiNimbleObjClass',
                                          fields = list(
-                                             ## nfObjectList = 'ANY',
                                              basePtrList = 'ANY',         ## List of pointers cast as derived C++ class
                                              namedObjectsPtrList = 'ANY', ## List of pointers cast as base C++ NamedObjects class
                                              neededObjectsList = 'ANY',
@@ -1250,8 +1287,21 @@ CmultiNimbleFunctionClass <- setRefClass('CmultiNimbleFunctionClass',
                                              show = function() {
                                                  cat(paste0('CmultiNimbleFunctionClass object\n'))
                                              },
-                                             initialize = function(compiledNodeFun, basePtrCall, project, ...) { ## need to set dll, nimbleProject
-                                                 callSuper(compiledNodeFun = compiledNodeFun, basePtrCall = basePtrCall, project = project, ...)
+                                             initialize = function(compiledNodeFun, basePtrCall,
+                                                                   project, ...) { ## need to set dll, nimbleProject
+                                                 callSuper(compiledNodeFun = compiledNodeFun,
+                                                           basePtrCall = basePtrCall,
+                                                           project = project, ...)
+                                                 boolCopyOneByOne <- !(as.character(cppCopyTypes) %in% c('nimbleFunction',
+                                                                                                         'nodeFxnVec',
+                                                                                                         'numericVector',
+                                                                                                         'doubleScalar',
+                                                                                                         'integerScalar',
+                                                                                                         'logicalScalar'))
+                                                 namesForCopyOneByOne <- names(cppCopyTypes)[ boolCopyOneByOne ]
+                                                 cppNamesOneByOne <<- cppNames[ cppNames %in% namesForCopyOneByOne ]
+                                                 cppCopyTypesOneByOne <<- cppCopyTypes[boolCopyOneByOne]
+                                                 
                                                  neededObjectsList <<- list()
                                                  basePtrList <<- list()
                                                  namedObjectsPtrList <<- list()
@@ -1276,12 +1326,16 @@ CmultiNimbleFunctionClass <- setRefClass('CmultiNimbleFunctionClass',
                                                  } else {
                                                      if(is.null(dll)) stop('In addInstance, DLL was not set and so must be provided when calling', call. = FALSE)
                                                      dll <<- dll       ## should only occur first time addInstance is called
+                                                     if(is.character(basePtrCall)) {
+                                                         updatedBasePtrCall <- try( getNativeSymbolInfo(basePtrCall, dll) )
+                                                         if(!inherits(updatedBasePtrCall, 'try-error'))
+                                                             basePtrCall <<- updatedBasePtrCall
+                                                     }
                                                  }
                                                  isNF <- is.nf(nfObject)
                                                  if(isNF) compiledNodeFun$nimCompProc$evalNewSetupLinesOneInstance(nfObject, check = TRUE)
                                                  # avoid R CMD check problem with registration:
-                                                 ##newBasePtr
-                                                 newObjPtrs <- eval(parse(text = ".Call(basePtrCall)", keep.source = FALSE))
+                                                 newObjPtrs <- eval(call('.Call', basePtrCall))
                                                  newBasePtr <- newObjPtrs[[1]] ## terminology confusing because this is the derived C++ class
                                                  newNamedObjectsPtr <- newObjPtrs[[ extPtrTypeIndex['NamedObjects'] ]] ## this is the base C++ class
                                                  if(is.null(newNamedObjectsPtr)) stop('Problem: Cannot find right external pointer information')
@@ -1293,7 +1347,7 @@ CmultiNimbleFunctionClass <- setRefClass('CmultiNimbleFunctionClass',
                                                                                                   
                                                  finalizationPtrList[[length(finalizationPtrList)+1]] <<- newNamedObjectsPtr
                                                  eval(call('.Call',nimbleUserNamespace$sessionSpecificDll$register_namedObjects_Finalizer,
-                                                           newNamedObjectsPtr, ##newBasePtr,
+                                                           newNamedObjectsPtr, 
                                                            dll[['handle']], regLabel))
                                                  
                                                  if(isNF) newRobject <- nimble:::nf_getRefClassObject(nfObject)
@@ -1303,7 +1357,14 @@ CmultiNimbleFunctionClass <- setRefClass('CmultiNimbleFunctionClass',
 
                                                  newNeededObjects <- nimbleInternalFunctions$buildNeededObjects(newRobject, compiledNodeFun, list(), dll, nimbleProject)
                                                  neededObjectsList[[length(neededObjectsList) + 1]] <<- newNeededObjects
-                                                 nimble:::copyFromRobject(newRobject, cppNames, cppCopyTypes, newNamedObjectsPtr, symTab = compiledNodeFun$nimCompProc$getSymbolTable(), dll) #newBasePtr, probably really need both for different cases
+                                                                                                  
+                                                 nimble:::copyFromRobject(newRobject,
+                                                                          cppNamesOneByOne,
+                                                                          cppCopyTypesOneByOne,
+                                                                          newNamedObjectsPtr,
+                                                                          symTab = compiledNodeFun$nimCompProc$getSymbolTable(),
+                                                                          dll,
+                                                                          useCompiledCopyMethod = TRUE) 
                                                  if(getNimbleOption('clearNimbleFunctionsAfterCompiling')) compiledNodeFun$nfProc$clearSetupOutputs(newRobject)
                                                  list(.self, length(basePtrList)) ## (this object, index)
                                              },
@@ -1329,7 +1390,6 @@ CmultiNimbleFunctionClass <- setRefClass('CmultiNimbleFunctionClass',
                                                  if(!(name %in% cppNames)) stop(paste0('Name ', name, ' is not a valid member variable in the requested object.', call.=FALSE))
                                                  basePtr <- basePtrList[[index]]
                                                  if(!inherits(basePtr, 'externalptr')) stop('Invalid index or basePtr', call. = FALSE)
-                                                 ##vptr <- nimbleInternalFunctions$newObjElementPtr(basePtr, name, dll = dll)
                                                  memberDataInternal(basePtr, index, name, value)
                                              }
                                          ))
@@ -1365,18 +1425,15 @@ CmultiNimbleListClass <- setRefClass('CmultiNimbleListClass',
                                                  } else {
                                                      if(is.null(dll)) stop('In addInstance, DLL was not set and so must be provided when calling', call. = FALSE)
                                                      dll <<- dll       ## should only occur first time addInstance is called
-                                                     ## experimental fix to handle predefined lists.  more general fix is to provide sessionSpecificDll as the dll
-                                                     ## natSymInfo <- try(getNativeSymbolInfo(compiledNodeFun$ptrCastFun$name, dll))
-                                                     ## if(inherits(natSymInfo, 'try-error')) {
-                                                     ##     natSymInfo <- try(getNativeSymbolInfo(compiledNodeFun$ptrCastFun$name))
-                                                     ##     if(inherits(natSymInfo, 'try-error')) {
-                                                     ##         stop(paste0('Problem loading compiled function ', compiledNodeFun$ptrCastFun$name))
-                                                     ##     }
-                                                     ## }
-                                                     ## castFunSymbolInfo <<- natSymInfo
+                                                     
+                                                     if(is.character(basePtrCall)) {
+                                                         updatedBasePtrCall <- try( getNativeSymbolInfo(basePtrCall, dll) )
+                                                         if(!inherits(updatedBasePtrCall, 'try-error'))
+                                                             basePtrCall <<- updatedBasePtrCall
+                                                     }
                                                      castFunSymbolInfo <<- getNativeSymbolInfo(compiledNodeFun$ptrCastFun$name, dll)
                                                  }
-                                                 newObjPtrs <- eval(parse(text = ".Call(basePtrCall)", keep.source = FALSE))
+                                                 newObjPtrs <-  eval(call('.Call', basePtrCall))
                                                  newPtrToSmartPtr <- newObjPtrs[[1]] ## terminology confusing because this is the derived C++ class
                                                  regLabel <- try(get('name', envir = nfObject), silent = TRUE)
                                                  if(inherits(regLabel, 'try-error') | is.null(regLabel)) regLabel <- environment(nfObject)$className
@@ -1388,13 +1445,19 @@ CmultiNimbleListClass <- setRefClass('CmultiNimbleListClass',
                                                  if(is.null(newFinalizationPtr)) stop('Problem: Cannot find right external pointer information')
                                                  finalizationPtrList[[length(finalizationPtrList)+1]] <<- newFinalizationPtr
                                                  eval(call('.Call',nimbleUserNamespace$sessionSpecificDll$register_smartPtrBase_Finalizer,
-                                                           newFinalizationPtr, ##.basePtr,
+                                                           newFinalizationPtr, 
                                                            dll[['handle']], regLabel))
                                                  newRobject <- nfObject
                                                  newRobject$.CobjectInterface <- list(.self, length(ptrToPtrList)) ## second element is its index
                                                  RobjectList[[length(RobjectList)+1]] <<- newRobject
                                                  namedObjectsPtr <- eval(call('.Call',castFunSymbolInfo, newPtrToPtr))
-                                                 nimble:::copyFromRobject(newRobject, cppNames, cppCopyTypes, namedObjectsPtr, symTab = compiledNodeFun$nimCompProc$getSymbolTable(), dll) #newNamedObjectsPointer was previously newBasePtr, probably really need both for different cases
+                                                 nimble:::copyFromRobject(newRobject,
+                                                                          cppNames,
+                                                                          cppCopyTypes,
+                                                                          namedObjectsPtr,
+                                                                          symTab = compiledNodeFun$nimCompProc$getSymbolTable(),
+                                                                          dll,
+                                                                          useCompiledCopyMethod = FALSE)
                                                  list(.self, length(ptrToSmartPtrList)) ## (this object, index)
                                              },
                                              clearInstance = function(index) { ## this is called when a Cmulti interface was built and later needs to be replaced with a full interface
